//////////////////////////////////////////////////////////////////////////////////////////////
// Clase para generar una geometra homognea en una dimensin                              //
//                                                                                          //
// Anlisis y Diseo y Programacin:                                                        //
//                                                                                          //
// Nombre:   Antonio Carrillo Ledesma                                                       //
// E-mail:   acl@www.mmc.geofisica.unam.mx                                                  //
// Pgina:   http://www.mmc.geofisica.unam.mx/acl                                           //
//                                                                                          //
//                                                                                          //
// Este programa es software libre. Puede redistribuirlo y/o modificarlo                    //
// bajo los trminos de la Licencia Pblica General de GNU segn es                         //
// publicada por la Free Software Foundation, bien de la versin 2 de                       //
// dicha Licencia o bien (segn su eleccin) de cualquier versin                           //
// posterior.                                                                               //
//                                                                                          //
// Este programa se distribuye con la esperanza de que sea til, pero SIN                   //
// NINGUNA GARANTA, incluso sin la garanta MERCANTIL implcita o sin                      //
// garantizar la CONVENIENCIA PARA UN PROPSITO PARTICULAR. Vase la                        //
// Licencia Pblica General de GNU para ms detalles.                                       //
//                                                                                          //
// Debera haber recibido una copia de la Licencia Pblica General junto                    //
// con este programa. Si no ha sido as, escriba a la Free Software                         //
// Foundation, Inc., en 675 Mass Ave, Cambridge, MA 02139, EEUU.                            //
//                                                                                          //
//                                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////////


#ifndef __Geometria_1DHomogenea__
#define __Geometria_1DHomogenea__

#include "Geometria_1D.hpp"

/// Clase para generar una geometra homognea en una dimensin
/** @author Antonio Carrillo Ledesma
    @date primavera 2009
    @version 1.0.1
    @bug No hay errores conocidos
*/
class Geometria_1DHomogenea : public Geometria_1D
{

public:

   /// Constructor de la clase
   /** @param xi Coordena inicial del dominio de la particin
       @param xf Coordena final del dominio de la particin
       @param part Tamao de la particin */
   Geometria_1DHomogenea(ldouble xi, ldouble xf, int part) : Geometria_1D()
   {
      Dominio.asigna(0,0,xi);
      Dominio.asigna(1,0,xf);

      Particion.asigna(0,part);

      N_N = part;
      N_E = part - 1;
      generaGeometria();
   }


   /// Constructor de la clase
   /** @param dom Dominio de la particion
       @param part Tamao de la particin */
   Geometria_1DHomogenea(MatrizDensa *dom, int part) : Geometria_1D()
   {
      dom->copia(&Dominio);

      Particion.asigna(0,part);

      N_N = part;
      N_E = part - 1;
      generaGeometria();
   }

   /// Constructor de la clase
   Geometria_1DHomogenea(void) : Geometria_1D()
   { }


   /// Constructor de la clase
   /** @param arch Nombre del archivo que contiene la geometria */
   Geometria_1DHomogenea(const char *arch) : Geometria_1D()
   {
      leeGeometria(arch);
   }


   /// Genera la geometria
   void generaGeometria(void);

};

/**
Esta clase implementa los componentes para el trabajar con una geometra homognea en una dimensin
@example EjemploGeometria1D.cpp */

#endif
