//////////////////////////////////////////////////////////////////////////////////////////////
// Clase para el trabajar con matrices de tamao variable                                   //
//                                                                                          //
// Anlisis y Diseo y Programacin:                                                        //
//                                                                                          //
// Nombre:   Antonio Carrillo Ledesma                                                       //
// E-mail:   acl@www.mmc.igeofcu.unam.mx                                                    //
// Pgina:   http://www.mmc.igeofcu.unam.mx/acl                                             //
//                                                                                          //
//                                                                                          //
// Este programa es software libre. Puede redistribuirlo y/o modificarlo                    //
// bajo los trminos de la Licencia Pblica General de GNU segn es                         //
// publicada por la Free Software Foundation, bien de la versin 2 de                       //
// dicha Licencia o bien (segn su eleccin) de cualquier versin                           //
// posterior.                                                                               //
//                                                                                          //
// Este programa se distribuye con la esperanza de que sea til, pero SIN                   //
// NINGUNA GARANTA, incluso sin la garanta MERCANTIL implcita o sin                      //
// garantizar la CONVENIENCIA PARA UN PROPSITO PARTICULAR. Vase la                        //
// Licencia Pblica General de GNU para ms detalles.                                       //
//                                                                                          //
// Debera haber recibido una copia de la Licencia Pblica General junto                    //
// con este programa. Si no ha sido as, escriba a la Free Software                         //
// Foundation, Inc., en 675 Mass Ave, Cambridge, MA 02139, EEUU.                            //
//                                                                                          //
//                                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////////



#ifndef __MatrizTamanoVariable__
#define __MatrizTamanoVariable__


#include "Matriz_Base.hpp"


/// Clase para el trabajar con matrices de tamao variable
/** @author Antonio Carrillo Ledesma
    @date primavera 2010
    @version 1.0.0
    @bug No hay errores conocidos
*/
class MatrizTamanoVariable : public Matriz_Base
{
private:

   /// Solicita la memoria necesaria para contener los valores de la matriz
   /** @param ren Nmero de renglones de la matriz
       @param col Nmero de columnas en cada renglon de la matriz  */
   void solicitaMemoria(const int ren, const int *col);


   /// Puntero a la matriz de datos
   ldouble **M;
   /// Arreglo que contiene el tamao de los renglones de la matriz
   int *TC;


public:

   /// Constructor de la clase
   /** @param ren Nmero de renglones de la matriz
       @param col Nmero de columnas en cada renglon de la matriz */
   MatrizTamanoVariable(const int ren, const int *col)
   {
      solicitaMemoria(ren, col);
      asignaNombre("SinNombre");
   }

   /// Constructor de la clase
   /** @param ren Nmero de renglones de la matriz
       @param col Nmero de columnas en cada renglon de la matriz
       @param nmb Nombre de la matriz */
   MatrizTamanoVariable(const int ren, const int *col, const char *nmb)
   {
      solicitaMemoria(ren, col);
      asignaNombre(nmb);
   }

   ~MatrizTamanoVariable()
   {
      liberaMemoria();
   }

   /// Libera la memoria solicitada para la matriz
   void liberaMemoria(void);

   /// Inicializa la matriz al valor indicado
   /** @param val Valor por omisin para inicializar la matriz */
   void inicializa(ldouble val);

#ifdef DEPURAR
   /// Asigna el valor indicado en el renglo y columna solicitado
   /** @param ren Renglon
       @param col Columna
       @param val Valor */
   void asigna(const int ren, const int col, const ldouble val);

   /// Retorna valor almacenado en el renglon y la columna indicada
   /** @param ren Nmero de renglon
       @param col Nmero de columna
       @return Valor almacenado en el renglon y la columna indicada  */
   ldouble retorna(const int ren, const int col);
#else
   /// Asigna el valor indicado en el renglo y columna solicitado
   /** @param ren Renglon
       @param col Columna
       @param val Valor */
   void asigna(const int ren, const int col, const ldouble val)
   {
      M[ren][col] = val;
   }

   /// Retorna valor almacenado en el renglon y la columna indicada
   /** @param ren Nmero de renglon
       @param col Nmero de columna
       @return Valor almacenado en el renglon y la columna indicada  */
   ldouble retorna(const int ren, const int col)
   {
      return (M[ren][col]);
   }
#endif

   /// Retorna el tamao del nmero de columna para dicho renglon
   /** @param ren Nmero de renglon
       @return Nmero de columnas del renglon */
   inline int retornaNumeroColumna(int ren)
   {
      return ( TC[ren]);
   }

   /// Copia el contenido de la matriz a la matriz pasada como parmetro
   /** @param a Puntero a una Matriz */
   void copia(MatrizTamanoVariable *a);

   /// Visualiza las matrices internas usadas para soportar a las matrices de tamao variable
   void visualizaMatricesInternas(void);

   /// Retorna el mximo tamao en todas las columnas que sean distintos de cero
   /** @return Mximo tamao de columnas ocupadas en la matriz */
   int maximoTamanoColumnaOcupado(void);

   /// Retorna el nmero de entradas distintas de cero en la matriz
   /** @return Nmero de entradas distintas de cero en la matriz */
   int entradasDistintasCero(void);

};


/**
Esta clase implementa los componentes para el trabajar con matrices de Tamao variable
@example EjemploTamanoVariable.cpp */

#endif
