//////////////////////////////////////////////////////////////////////////////////////////////
// Clase para generar una geometra heterognea en una dimensin                            //
//                                                                                          //
// Anlisis y Diseo y Programacin:                                                        //
//                                                                                          //
// Nombre:   Antonio Carrillo Ledesma                                                       //
// E-mail:   acl@www.mmc.geofisica.unam.mx                                                  //
// Pgina:   http://www.mmc.geofisica.unam.mx/acl                                           //
//                                                                                          //
//                                                                                          //
// Este programa es software libre. Puede redistribuirlo y/o modificarlo                    //
// bajo los trminos de la Licencia Pblica General de GNU segn es                         //
// publicada por la Free Software Foundation, bien de la versin 2 de                       //
// dicha Licencia o bien (segn su eleccin) de cualquier versin                           //
// posterior.                                                                               //
//                                                                                          //
// Este programa se distribuye con la esperanza de que sea til, pero SIN                   //
// NINGUNA GARANTA, incluso sin la garanta MERCANTIL implcita o sin                      //
// garantizar la CONVENIENCIA PARA UN PROPSITO PARTICULAR. Vase la                        //
// Licencia Pblica General de GNU para ms detalles.                                       //
//                                                                                          //
// Debera haber recibido una copia de la Licencia Pblica General junto                    //
// con este programa. Si no ha sido as, escriba a la Free Software                         //
// Foundation, Inc., en 675 Mass Ave, Cambridge, MA 02139, EEUU.                            //
//                                                                                          //
//                                                                                          //
//////////////////////////////////////////////////////////////////////////////////////////////


#ifndef __Geometria_1DHeterogenea__
#define __Geometria_1DHeterogenea__


#include "Geometria_1D.hpp"


/// Tipo de justificacin de la malla
#define IZQUIERDA  -1
#define CENTRADA    0
#define DERECHA     1


/// Tipo de Malla
#define DOMINIO_NM     1   // Dominio/(n^m) 
#define DOMINIO_LOGe   2   // Dominio/Loge
#define DIMINIO_LOG10  3   // Dominio/Log10



/// Clase para generar una geometra heterognea en una dimensin
/** @author Antonio Carrillo Ledesma
    @date primavera 2009
    @version 1.0.1
    @bug No hay errores conocidos
    @todo Implementar diversos tipos de geometra no homognea
*/
class Geometria_1DHeterogenea : public Geometria_1D
{
private:

   /// Tipo de geometra
   int Tipo;
   /// Indica el tipo de justificacin de la geometra
   int Centrada;
   /// Exponente para geometras del tipo 1/2^M
   int M;

   /// Dominio del tipo 1/N^M
   void dominioNM(void);

   /// Dominio del logartmico
   void dominioLOGe(void);

   /// Dominio del logartmico
   void dominioLOG10(void);


public:

   /** @param xi Coordena inicial del dominio de la particin
       @param xf Coordena final del dominio de la particin
       @param part Tamao de la particin
       @param tp Tipo de particin
       @param cen Particin centrada en el punto indicado */
   Geometria_1DHeterogenea(ldouble xi, ldouble xf, int part, int tp, double cen=0.0) : Geometria_1D()
   {
      Dominio.asigna(0,0,xi);
      Dominio.asigna(1,0,xf);

      Particion.asigna(0,part);

      N_N = part;
      N_E = part - 1;

      Tipo = tp;
      Centrada = cen;
      M = 2;

      generaGeometria();
   }

   /// Constructor de la clase
   /** @param dom Dominio de la particion
       @param part Tamao de la particin
       @param tp Tipo de particin
       @param cen Particin centrada en el punto indicado */
   Geometria_1DHeterogenea(MatrizDensa *dom, int part, int tp, double cen=0.0) : Geometria_1D()
   {
      dom->copia(&Dominio);

      Particion.asigna(0,part);

      N_N = part;
      N_E = part - 1;

      Tipo = tp;
      Centrada = cen;

      generaGeometria();
   }


   /// Constructor de la clase
   /** @param arch Nombre del archivo que contiene la geometria */
   Geometria_1DHeterogenea(char *arch) : Geometria_1D()
   {
      leeGeometria(arch);
   }


   /// Genera la geometria
   void generaGeometria(void);



};

/**
Esta clase implementa los componentes para el trabajar con una geometra heterognea en una dimensin
@example EjemploGeometria1D.cpp */

#endif
